@@ -15,9 +15,7 @@
    Library General Public License for more details.
   
    You should have received a copy of the GNU Library General Public
-   License along with this program; if not, write to the
-   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   License along with this program; if not, see <http://www.gnu.org/licenses/>.
  
    Authors: Andy Hertzfeld <andy@eazel.com>
  */
