@@ -16,8 +16,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Authors: Darin Adler <darin@bentspoon.com>
  */
@@ -85,6 +84,11 @@ nautilus_desktop_window_constructed (GObject *obj)
 					      NAUTILUS_ACTION_NEW_TAB);
 	gtk_action_set_sensitive (action, FALSE);
 
+	/* Don't allow search on desktop */
+	action = gtk_action_group_get_action (action_group,
+					      NAUTILUS_ACTION_SEARCH);
+	gtk_action_set_sensitive (action, FALSE);
+
 	/* Set the accessible name so that it doesn't inherit the cryptic desktop URI. */
 	accessible = gtk_widget_get_accessible (GTK_WIDGET (window));
 
