@@ -17,8 +17,7 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; see the file COPYING.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * see <http://www.gnu.org/licenses/>.
  *
  * Author: Maciej Stachowiak <mjs@eazel.com>
  *         Ettore Perazzoli <ettore@gnu.org>
@@ -656,19 +655,24 @@ void
 nautilus_location_entry_set_secondary_action (NautilusLocationEntry *entry,
 					      NautilusLocationEntryAction secondary_action)
 {
+	gboolean rtl;
+
 	if (entry->details->secondary_action == secondary_action) {
 		return;
 	}
+
+	rtl = gtk_widget_get_direction (GTK_WIDGET (entry)) == GTK_TEXT_DIR_RTL;
+
 	switch (secondary_action) {
 	case NAUTILUS_LOCATION_ENTRY_ACTION_CLEAR:
 		gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry), 
 						   GTK_ENTRY_ICON_SECONDARY,
-						   "edit-clear-symbolic");
+						   rtl ? "edit-clear-rtl-symbolic" : "edit-clear-symbolic");
 		break;
 	case NAUTILUS_LOCATION_ENTRY_ACTION_GOTO:
 		gtk_entry_set_icon_from_icon_name (GTK_ENTRY (entry),
 						   GTK_ENTRY_ICON_SECONDARY,
-						   "go-next-symbolic");
+						   rtl ? "go-next-rtl-symbolic" : "go-next-symbolic");
 		break;
 	default:
 		g_assert_not_reached ();
@@ -753,7 +757,7 @@ nautilus_location_entry_new (void)
 {
 	GtkWidget *entry;
 
-	entry = gtk_widget_new (NAUTILUS_TYPE_LOCATION_ENTRY, NULL);
+	entry = gtk_widget_new (NAUTILUS_TYPE_LOCATION_ENTRY, "max-width-chars", 350, NULL);
 
 	return entry;
 }
